var renderClass = "jp.ngt.rtm.render.VehiclePartsRenderer";
importPackage(Packages.org.lwjgl.opengl);
importPackage(Packages.jp.ngt.rtm.render);
importPackage(Packages.jp.ngt.ngtlib.math);
importPackage(Packages.jp.ngt.rtm);
importPackage(Packages.jp.ngt.rtm.entity.train.util);
importPackage(Packages.jp.ngt.ngtlib.io);
importPackage(Packages.jp.ngt.rtm.entity.vehicle);
importPackage(Packages.jp.ngt.ngtlib.util);
importPackage(Packages.jp.ngt.ngtlib.renderer);
//データ保存処理(グローバル関数)
var LocalData = new java.util.HashMap();

//##### 初期化 ###########################
function init(par1, par2) {
	tick = false;

	body = renderer.registerParts(new Parts("cabroof", "number", "head", "frame", "deflecter", "boiler", "parts", "pipe", "chimney", "cab", "underparts"));

	lightF = renderer.registerParts(new Parts("light-F"));
	lightB = renderer.registerParts(new Parts("light-B"));

	wheel_f = renderer.registerParts(new Parts("frontwheel"));
	bogie_f = renderer.registerParts(new Parts("frontbogie"));
	wheel_m1 = renderer.registerParts(new Parts("wheel1"));
	wheel_m2 = renderer.registerParts(new Parts("wheel2"));
	wheel_m3 = renderer.registerParts(new Parts("wheel3"));

	rod1R = renderer.registerParts(new Parts("rod1R")); //サイドロッド
	rod2R = renderer.registerParts(new Parts("rod2R")); //メインロッド
	rod3R = renderer.registerParts(new Parts("rod3R")); //ピストン・結びリンク
	rod4R = renderer.registerParts(new Parts("rod4R")); //合併テコ
	rod5R = renderer.registerParts(new Parts("rod5R")); //加減リンク
	rod6R = renderer.registerParts(new Parts("rod6R")); //エキセントリックロッド
	rod1L = renderer.registerParts(new Parts("rod1L"));
	rod2L = renderer.registerParts(new Parts("rod2L"));
	rod3L = renderer.registerParts(new Parts("rod3L"));
	rod4L = renderer.registerParts(new Parts("rod4L"));
	rod5L = renderer.registerParts(new Parts("rod5L"));
	rod6L = renderer.registerParts(new Parts("rod6L"));

	//バージョン確認
	MCVer = MCVersionChecker();

	//ロッド設定
	rodSet = {};
	rodSet.r1 = 0.3027, //メインロッド基部径
		rodSet.r2 = 0.1226, //リターンクランク径
		rodSet.r3 = 0.3060, //加減リンク径
		rodSet.c1 = 2.2834, //メインロッド長
		rodSet.c2 = 0.880136, //合併テコ長
		rodSet.v1 = 21.000; //加減リンク動作角
	pvWheel = [-0.200, 0.8312, -1.0738, -3.3598]; //動輪軸Y値,動輪1Z値,動輪2Z値,動輪3Z値
	pvRod4 = [-0.5027, 1.2826]; //合併テコ原点Y・Z値
	pvRod5 = [0.2185, 0.2398]; //加減リンク原点Y・Z値

	//台車設定
	bogieSet = {};
	bogieSet.bd0 = 0.95; //json台車間距離(台車は前後対称位置とする)
	bogieSet.bd2 = -3.3598; //後台車実位置（表示用）
	bogieSet.wd0 = 2.69; //軸距÷2(前台車)
	bogieSet.wd1 = 1.00; //軸距÷2(後台車)
	bogieSet.wy0 = -0.515; //車輪軸Y座標(前後台車)

	//modelPreview
	trainDistance = 4.95;
	//viewPanel = true;
}



//##### MCバージョンチェック ###########################
function MCVersionChecker() {
	var version = RTMCore.VERSION;
	if (version.indexOf("1.7.10") >= 0) return "1.7.10";
	else if (version.indexOf("2.0") >= 0) return "1.8.9";
	else if (version.indexOf("2.1") >= 0) return "1.9.4";
	else if (version.indexOf("2.2") >= 0) return "1.10.2";
	else if (version.indexOf("2.4") >= 0) return "1.12.2";
	else return "unknown";
}

//##### 号車取得 ####################
function isMiddleCar(entity) {//trueなら中間車、falseなら先頭車/後尾車

	//-通さない処理-
	if (entity === null) return;
	//if (renderer.currentMatId !== 5) return;//材質でも走査されるため

	var data = LocalData.get(entity);//保存された"データ群"の呼び出し
	var state = false;

	var formation = entity.getFormation();//formation型
	if (formation == null) {//formationが取得できないときの処理
		state = data.prevstate;
		if (data.prevstate == null) state = false;
	} else {
		var current = formation.getEntry(entity).entryId + 1;//この車両の号車を取得。0から数えるため+1する
		var max = formation.size();//連結してる両数を返す
		if (current == 1 || current == max) {//この車両が先頭or最後尾なら
			state = false;
		} else {//そうでない(つまり中間)なら
			state = true;
		}
	}

	data.prevstate = state;
	LocalData.put(entity, data);

	return state;
}


//##### 車両情報取得 ###########################
function TKMTS(entity, par1) {
	var r = 0.0;
	if (!entity) return r;

	switch (par1) {
		case "speed": r = entity.getSpeed() * 72.0; break;
		case "notch": r = entity.getNotch(); break;
		case "entityID": r = entity.func_145782_y(); break;
		case "tick": r = renderer.getTick(entity); break;
		case "formationNum": r = entity.getFormation().getEntry(entity).entryId + 1; break;
		default:
			if (MCVer == "1.7.10" || MCVer == "1.8.9" || MCVer == "1.9.4") {
				switch (par1) {
					case "trainDir": r = entity.getTrainStateData(0); break;
					case "signal": r = entity.getTrainStateData(2); break;
					case "destination": r = entity.getTrainStateData(8); break;
					case "announce": r = entity.getTrainStateData(9); break;
					case "direction": r = entity.getTrainStateData(10); break;
				}
			} else {
				switch (par1) {
					case "trainDir": r = entity.getVehicleState(TrainState.getStateType(0)); break;
					case "signal": r = entity.getVehicleState(TrainState.getStateType(2)); break;
					case "destination": r = entity.getVehicleState(TrainState.getStateType(8)); break;
					case "announce": r = entity.getVehicleState(TrainState.getStateType(9)); break;
					case "direction": r = entity.getVehicleState(TrainState.getStateType(10)); break;
				}
			}
	}

	return r;
}



//##### tick管理 #############################
function detectTick(entity) {
	//initでtick=false初期化
	var dataMap = entity.getResourceState().getDataMap(),
		tick = TKMTS(entity, "tick"),
		prevTick = dataMap.getInt("tick");
	if (tick != prevTick) {
		dataMap.setInt("tick", tick, false);
		return true;
	} else {
		return false;
	}
}



//##### render #####################
function render(entity, pass, par3) {
	var data = LocalData.get(entity) || {};
	LocalData.put(entity, data);

	//UpdateTick
	if (entity != null) tick = detectTick(entity);

	GL11.glPushMatrix();

	//modelPreview(entity);
	var Middle = isMiddleCar(entity);

	GL11.glTranslatef(0.0, 0.0, 0.0); //接地高補正

	//通常描画
	if (pass == 0) {
		body.render(renderer);
		render_bogie(entity, tick);
		render_light(entity, Middle, pass);
		render_rod(entity, tick);
	}
	//半透明描画
	if (pass == 1) {
		//AlphaBrightFix
		if (MCVer != "1.7.10") Packages.jp.ngt.ngtlib.renderer.GLHelper.setBrightness(entity ? entity.func_70070_b() : 0);

		//alpha.render(renderer);
	}

	//ライト描画
	if (pass > 1) {
		render_light(entity, Middle, pass);
	}

	GL11.glPopMatrix();
}



//##### 車輪回転 #############################
function wheelRotate(entity, tick, wheelName, diameter) {

	if (entity == null) return new Date().getTime() / 4 % 3600;

	var dataMap = entity.getResourceState().getDataMap();
	wheelName = String(wheelName) + "_wheelRotate";
	var wheelAnglePrev = dataMap.getDouble(wheelName);

	if (tick) {
		var speed = TKMTS(entity, "speed") * (TKMTS(entity, "trainDir") * -2 + 1);
		var runDistance = speed * 1000 / 60 / 60 / 20; //走行m/tick
		var wheelRotate = runDistance / ((1 + diameter) * 2 * Math.PI) * 360; //回転度/tick
		var wheelAngle = (wheelAnglePrev + wheelRotate) % 360;

		dataMap.setDouble(wheelName, wheelAngle, false);
	} else {
		var wheelAngle = wheelAnglePrev;
	}

	return wheelAngle;
}



//##### ロッド動作 ###########################
function render_rod(entity, tick) {

	var f0 = wheelRotate(entity, tick, "mainWheel", 1 + pvWheel[0]);
	var radL = NGTMath.toRadians(f0);
	var hL = NGTMath.getSin(radL) * rodSet["r1"];
	var xL = Math.sqrt(rodSet["c1"] * rodSet["c1"] - hL * hL);
	var roRod2L = NGTMath.getSin(-radL / 2.0) - NGTMath.toDegrees(Math.atan2(hL, xL));
	var stroke3L = 2.0 * rodSet["r1"] - (rodSet["c1"] + rodSet["r1"] - (xL - NGTMath.getCos(radL) * rodSet["r1"]));
	var roRod4L = NGTMath.toDegrees(Math.atan2(rodSet["c2"], stroke3L)) - 90.0;
	var roRod5L = NGTMath.getSin(radL) * rodSet["v1"];
	var moRod5L = NGTMath.toRadians(NGTMath.getSin(radL) * rodSet["v1"]);
	var xrodZL = (pvRod5[1] - pvWheel[2]) - (NGTMath.getSin(moRod5L) * rodSet["r3"]) - (NGTMath.getCos(NGTMath.toRadians(f0 + 90)) * rodSet["r2"]);
	var xrodYL = (pvRod5[0] - pvWheel[0]) - (NGTMath.getCos(moRod5L) * rodSet["r3"]) - (NGTMath.getSin(NGTMath.toRadians(f0 + 270)) * rodSet["r2"]);
	var roRod6L = NGTMath.toDegrees(Math.atan2(xrodYL, xrodZL));

	var f0R = (f0 + 90.0) % 360;
	var radR = NGTMath.toRadians(f0R);
	var hR = NGTMath.getSin(radR) * rodSet["r1"];
	var xR = Math.sqrt(rodSet["c1"] * rodSet["c1"] - hR * hR);
	var roRod2R = NGTMath.getSin(-radR / 2.0) - NGTMath.toDegrees(Math.atan2(hR, xR));
	var stroke3R = 2.0 * rodSet["r1"] - (rodSet["c1"] + rodSet["r1"] - (xR - NGTMath.getCos(radR) * rodSet["r1"]));
	var roRod4R = NGTMath.toDegrees(Math.atan2(rodSet["c2"], stroke3R)) - 90.0;
	var roRod5R = NGTMath.getSin(radR) * rodSet["v1"];
	var moRod5R = NGTMath.toRadians(NGTMath.getSin(radR) * rodSet["v1"]);
	var xrodZR = (pvRod5[1] - pvWheel[2]) - (NGTMath.getSin(moRod5R) * rodSet["r3"]) - (NGTMath.getCos(NGTMath.toRadians(f0R + 90)) * rodSet["r2"]);
	var xrodYR = (pvRod5[0] - pvWheel[0]) - (NGTMath.getCos(moRod5R) * rodSet["r3"]) - (NGTMath.getSin(NGTMath.toRadians(f0R + 270)) * rodSet["r2"]);
	var roRod6R = NGTMath.toDegrees(Math.atan2(xrodYR, xrodZR));

	GL11.glPushMatrix();

	//動輪１
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, pvWheel[0], pvWheel[1]);
	wheel_m1.render(renderer);
	GL11.glPopMatrix();
	//動輪２
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, pvWheel[0], pvWheel[2]);
	wheel_m2.render(renderer);
	GL11.glPopMatrix();
	//動輪３
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, pvWheel[0], pvWheel[3]);
	wheel_m3.render(renderer);
	GL11.glPopMatrix();

	//左サイドロッド
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, pvWheel[0], pvWheel[2]);
	renderer.rotate(-f0, 'X', 0.0, pvWheel[0], pvWheel[2] - rodSet["r1"]);
	rod1L.render(renderer);
	//左メインロッド
	renderer.rotate(-roRod2L, 'X', 0.0, pvWheel[0], pvWheel[2] - rodSet["r1"]);
	rod2L.render(renderer);
	GL11.glPopMatrix();
	//左ピストン
	GL11.glPushMatrix();
	GL11.glTranslatef(0.0, 0.0, stroke3L);
	rod3L.render(renderer);
	//左合併テコ
	renderer.rotate(roRod4L, 'X', 0.0, pvRod4[0], pvRod4[1]);
	rod4L.render(renderer);
	GL11.glPopMatrix();
	//左加減リンク
	GL11.glPushMatrix();
	renderer.rotate(roRod5L, 'X', 0.0, pvRod5[0], pvRod5[1]);
	rod5L.render(renderer);
	GL11.glPopMatrix();
	//左エキセントリックロッド
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, pvWheel[0], pvWheel[2]);
	renderer.rotate(-f0, 'X', 0.0, pvWheel[0] - rodSet["r2"], pvWheel[2]);
	renderer.rotate(-roRod6L, 'X', 0.0, pvWheel[0] - rodSet["r2"], pvWheel[2]);
	rod6L.render(renderer);
	GL11.glPopMatrix();

	//右サイドロッド
	GL11.glPushMatrix();
	renderer.rotate(f0R, 'X', 0.0, pvWheel[0], pvWheel[2]);
	renderer.rotate(-f0R, 'X', 0.0, pvWheel[0], pvWheel[2] - rodSet["r1"]);
	rod1R.render(renderer);
	//右メインロッド
	renderer.rotate(-roRod2R, 'X', 0.0, pvWheel[0], pvWheel[2] - rodSet["r1"]);
	rod2R.render(renderer);
	GL11.glPopMatrix();
	//右ピストン
	GL11.glPushMatrix();
	GL11.glTranslatef(0.0, 0.0, stroke3R);
	rod3R.render(renderer);
	//右合併テコ
	renderer.rotate(roRod4R, 'X', 0.0, pvRod4[0], pvRod4[1]);
	rod4R.render(renderer);
	GL11.glPopMatrix();
	//右加減リンク
	GL11.glPushMatrix();
	renderer.rotate(roRod5R, 'X', 0.0, pvRod5[0], pvRod5[1]);
	rod5R.render(renderer);
	GL11.glPopMatrix();
	//右エキセントリックロッド
	GL11.glPushMatrix();
	renderer.rotate(f0R, 'X', 0.0, pvWheel[0], pvWheel[2]);
	renderer.rotate(-f0R, 'X', 0.0, pvWheel[0] - rodSet["r2"], pvWheel[2]);
	renderer.rotate(-roRod6R, 'X', 0.0, pvWheel[0] - rodSet["r2"], pvWheel[2]);
	rod6R.render(renderer);
	GL11.glPopMatrix();

	GL11.glPopMatrix();
}


////##### render.台車 ########################
function render_bogie(entity, tick) {

	var f0 = wheelRotate(entity, tick, "subWheel", 1 + bogieSet["wy0"]),
		bodyYaw = 0.0,
		bogieFYaw = 0.0,
		bodyPitch = 0.0,
		bogieFPitch = 0.0;

	if (entity != null && entity.getBogie(0) != null && entity.getBogie(1) != null) {

		bodyYaw = entity.field_70177_z;

		//前台車ヨーローカル角
		var i = entity.getBogie(0).field_70177_z - bodyYaw;
		if (i > 180) {
			bogieFYaw = i - 360;
		} else if (i < -180) {
			bogieFYaw = i + 360;
		} else {
			bogieFYaw = i;
		}
		//ピッチローカル角
		bodyPitch = entity.field_70125_A;
		bogieFPitch = entity.getBogie(0).field_70125_A * -1 + bodyPitch;
	}

	//前台車
	GL11.glPushMatrix();
	renderer.rotate(bogieFYaw, 'Y', 0.0, 0.0, bogieSet["bd0"]);
	renderer.rotate(bogieFPitch, 'X', 0.0, 0.0, bogieSet["bd0"]);
	bogie_f.render(renderer);
		GL11.glPushMatrix();
		renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], bogieSet["bd0"] + bogieSet["wd0"]);
		wheel_f.render(renderer);
		GL11.glPopMatrix();
	GL11.glPopMatrix();

}

//##### render_ライト ####################
function render_light(entity, Middle, pass) {

	var version = MCVersionChecker();
	var lightMove = 0;


	if (entity != null) {
		if (version == "1.7.10" || version == "1.8.9" || version == "1.9.4") {
			lightMove = entity.getTrainStateData(0);
		} else {
			lightMove = entity.getVehicleState(TrainState.getStateType(0));
		}

	}


	GL11.glPushMatrix();
	if (Middle == false) {



		if (lightMove == 0) {
			GL11.glPushMatrix();
			lightF.render(renderer);
			GL11.glPopMatrix();
		} else {
			GL11.glPushMatrix();
			lightB.render(renderer);
			GL11.glPopMatrix();
		}

	} else {


		if (pass == 0) {
			GL11.glPushMatrix();
			lightB.render(renderer);
			GL11.glPopMatrix();
		}

	}
	GL11.glPopMatrix();
}
