var renderClass = "jp.ngt.rtm.render.VehiclePartsRenderer";
importPackage(Packages.org.lwjgl.opengl);
importPackage(Packages.jp.ngt.rtm.render);
importPackage(Packages.jp.ngt.ngtlib.math);
importPackage(Packages.jp.ngt.rtm);
importPackage(Packages.jp.ngt.rtm.entity.train.util);
importPackage(Packages.jp.ngt.ngtlib.io);
importPackage(Packages.jp.ngt.rtm.entity.vehicle);
importPackage(Packages.jp.ngt.ngtlib.util);
importPackage(Packages.jp.ngt.ngtlib.renderer);
//データ保存処理(グローバル関数)
var LocalData = new java.util.HashMap();

//##### 初期化 ###########################
function init(par1, par2) {
	tick = false;

	body = renderer.registerParts(new Parts("body"));

	lightF = renderer.registerParts(new Parts("light-F"));
	lightB = renderer.registerParts(new Parts("light-B"));

	wheela = renderer.registerParts(new Parts("wheela"));
	wheelb = renderer.registerParts(new Parts("wheelb"));
	wheelc = renderer.registerParts(new Parts("wheelc"));
	//バージョン確認
	MCVer = MCVersionChecker();

	//台車設定
	bogieSet = {};
	bogieSet.wy0 = -0.53; //車輪軸Y座標(前後台車)

	//modelPreview
	trainDistance = 3.45;
}



//##### MCバージョンチェック ###########################
function MCVersionChecker() {
	var version = RTMCore.VERSION;
	if (version.indexOf("1.7.10") >= 0) return "1.7.10";
	else if (version.indexOf("2.0") >= 0) return "1.8.9";
	else if (version.indexOf("2.1") >= 0) return "1.9.4";
	else if (version.indexOf("2.2") >= 0) return "1.10.2";
	else if (version.indexOf("2.4") >= 0) return "1.12.2";
	else return "unknown";
}

//##### 号車取得 ####################
function isMiddleCar(entity) {//trueなら中間車、falseなら先頭車/後尾車

	//-通さない処理-
	if (entity === null) return;
	//if (renderer.currentMatId !== 5) return;//材質でも走査されるため

	var data = LocalData.get(entity);//保存された"データ群"の呼び出し
	var state = false;

	var formation = entity.getFormation();//formation型
	if (formation == null) {//formationが取得できないときの処理
		state = data.prevstate;
		if (data.prevstate == null) state = false;
	} else {
		var current = formation.getEntry(entity).entryId + 1;//この車両の号車を取得。0から数えるため+1する
		var max = formation.size();//連結してる両数を返す
		if (current == 1 || current == max) {//この車両が先頭or最後尾なら
			state = false;
		} else {//そうでない(つまり中間)なら
			state = true;
		}
	}

	data.prevstate = state;
	LocalData.put(entity, data);

	return state;
}


//##### 車両情報取得 ###########################
function TKMTS(entity, par1) {
	var r = 0.0;
	if (!entity) return r;

	switch (par1) {
		case "speed": r = entity.getSpeed() * 72.0; break;
		case "notch": r = entity.getNotch(); break;
		case "entityID": r = entity.func_145782_y(); break;
		case "tick": r = renderer.getTick(entity); break;
		case "formationNum": r = entity.getFormation().getEntry(entity).entryId + 1; break;
		default:
			if (MCVer == "1.7.10" || MCVer == "1.8.9" || MCVer == "1.9.4") {
				switch (par1) {
					case "trainDir": r = entity.getTrainStateData(0); break;
					case "signal": r = entity.getTrainStateData(2); break;
					case "destination": r = entity.getTrainStateData(8); break;
					case "announce": r = entity.getTrainStateData(9); break;
					case "direction": r = entity.getTrainStateData(10); break;
				}
			} else {
				switch (par1) {
					case "trainDir": r = entity.getVehicleState(TrainState.getStateType(0)); break;
					case "signal": r = entity.getVehicleState(TrainState.getStateType(2)); break;
					case "destination": r = entity.getVehicleState(TrainState.getStateType(8)); break;
					case "announce": r = entity.getVehicleState(TrainState.getStateType(9)); break;
					case "direction": r = entity.getVehicleState(TrainState.getStateType(10)); break;
				}
			}
	}

	return r;
}



//##### tick管理 #############################
function detectTick(entity) {
	//initでtick=false初期化
	var dataMap = entity.getResourceState().getDataMap(),
		tick = TKMTS(entity, "tick"),
		prevTick = dataMap.getInt("tick");
	if (tick != prevTick) {
		dataMap.setInt("tick", tick, false);
		return true;
	} else {
		return false;
	}
}



//##### render #####################
function render(entity, pass, par3) {
	var data = LocalData.get(entity) || {};
	LocalData.put(entity, data);

	//UpdateTick
	if (entity != null) tick = detectTick(entity);

	GL11.glPushMatrix();

	//modelPreview(entity);
	var Middle = isMiddleCar(entity);

	GL11.glTranslatef(0.0, 0.0, 0.0); //接地高補正

	//通常描画
	if (pass == 0) {
		body.render(renderer);
		render_bogie(entity, tick);
		render_light(entity, Middle, pass);
	}
	//半透明描画
	if (pass == 1) {
		if (MCVer != "1.7.10") Packages.jp.ngt.ngtlib.renderer.GLHelper.setBrightness(entity ? entity.func_70070_b() : 0);

		//alpha.render(renderer);
	}

	//ライト描画
	if (pass > 1) {
		render_light(entity, Middle, pass);
	}

	GL11.glPopMatrix();
}



//##### 車輪回転 #############################
function wheelRotate(entity, tick, wheelName, diameter) {

	if (entity == null) return new Date().getTime() / 4 % 3600;

	var dataMap = entity.getResourceState().getDataMap();
	wheelName = String(wheelName) + "_wheelRotate";
	var wheelAnglePrev = dataMap.getDouble(wheelName);

	if (tick) {
		var speed = TKMTS(entity, "speed") * (TKMTS(entity, "trainDir") * -2 + 1);
		var runDistance = speed * 1000 / 60 / 60 / 20; //走行m/tick
		var wheelRotate = runDistance / ((1 + diameter) * 2 * Math.PI) * 360; //回転度/tick
		var wheelAngle = (wheelAnglePrev + wheelRotate) % 360;

		dataMap.setDouble(wheelName, wheelAngle, false);
	} else {
		var wheelAngle = wheelAnglePrev;
	}

	return wheelAngle;
}



////##### render.台車 ########################
function render_bogie(entity, tick) {

	var f0 = wheelRotate(entity, tick, "subWheel", 1 + bogieSet["wy0"]),
		bodyYaw = 0.0,
		bogieFYaw = 0.0,
		bodyPitch = 0.0,
		bogieFPitch = 0.0;

	if (entity != null && entity.getBogie(0) != null && entity.getBogie(1) != null) {

		bodyYaw = entity.field_70177_z;

		//前台車ヨーローカル角
		var i = entity.getBogie(0).field_70177_z - bodyYaw;
		if (i > 180) {
			bogieFYaw = i - 360;
		} else if (i < -180) {
			bogieFYaw = i + 360;
		} else {
			bogieFYaw = i;
		}
		//ピッチローカル角
		bodyPitch = entity.field_70125_A;
		bogieFPitch = entity.getBogie(0).field_70125_A * -1 + bodyPitch;
	}

	//前台車
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], 1.5138);
	wheela.render(renderer);
	GL11.glPopMatrix();
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], -0.3152);
	wheelb.render(renderer);
	GL11.glPopMatrix();
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], -2.1442);
	wheelc.render(renderer);
	GL11.glPopMatrix();

}

//##### render_ライト ####################
function render_light(entity, Middle, pass) {

	var version = MCVersionChecker();
	var lightMove = 0;


	if (entity != null) {
		if (version == "1.7.10" || version == "1.8.9" || version == "1.9.4") {
			lightMove = entity.getTrainStateData(0);
		} else {
			lightMove = entity.getVehicleState(TrainState.getStateType(0));
		}

	}


	GL11.glPushMatrix();
	if (Middle == false) {



		if (lightMove == 0) {
			GL11.glPushMatrix();
			lightF.render(renderer);
			GL11.glPopMatrix();
		} else {
			GL11.glPushMatrix();
			lightB.render(renderer);
			GL11.glPopMatrix();
		}

	} else {


		if (pass == 0) {
			GL11.glPushMatrix();
			lightB.render(renderer);
			GL11.glPopMatrix();
		}

	}
	GL11.glPopMatrix();
}
